<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
  xmlns:template="http://www.boddie.org.uk/ns/xmltools/template">

  <xsl:output indent="yes"/>
  <xsl:namespace-alias stylesheet-prefix="axsl" result-prefix="xsl"/>



  <!-- Match the document itself. -->

  <xsl:template match="/">
    <axsl:stylesheet version="1.0">

      <axsl:output indent="yes"/>

      <!-- Make document parameters for all elements appearing to use enumerations. -->
      <xsl:for-each select="//element[*/@expr]">
        <axsl:param name="{@name}"/>
      </xsl:for-each>

      <!-- Make a document-level rule. -->
      <axsl:template match="/">
        <axsl:apply-templates select="*"/>
      </axsl:template>

      <!-- Process the top-level element to make other rules. -->
      <xsl:apply-templates select="*"/>

      <!-- Replicate unknown elements. -->
      <axsl:template match="@*|node()">
        <axsl:copy>
          <axsl:apply-templates select="@*|node()"/>
        </axsl:copy>
      </axsl:template>

    </axsl:stylesheet>
  </xsl:template>



  <!-- Match element references. -->

  <xsl:template match="element">

    <!-- Make a rule for the element. -->
    <axsl:template match="{@name}">

      <!-- Copy the element. -->
      <xsl:element name="{@name}">

        <!-- Process attributes. -->
        <axsl:apply-templates select="@*"/>

        <!-- Find elements and determine how to process them. -->
        <xsl:for-each select="element">
          <xsl:call-template name="process-element"/>
        </xsl:for-each>
      </xsl:element>
    </axsl:template>

    <!-- Make rules for nested elements. -->
    <xsl:for-each select="element">
      <xsl:call-template name="process-rules"/>
    </xsl:for-each>

  </xsl:template>



  <!-- Process elements. -->

  <xsl:template name="process-element">
    <xsl:param name="path" select="@name"/>

    <!-- Define elements which do not have selectors. -->
    <xsl:variable name="adding-selectors" select="count(//selector[@element=current()/@name])"/>

    <xsl:choose>
      <!-- Enumerations. -->
      <xsl:when test="@type='multiple-choice-value' or @type='multiple-choice-list-value'">
        <xsl:call-template name="inside-enumeration">
          <xsl:with-param name="path" select="$path"/>
        </xsl:call-template>
      </xsl:when>
      <!-- Added elements. -->
      <xsl:when test="$adding-selectors = 0">
        <xsl:element name="{@name}">
          <axsl:apply-templates select="{$path}/@*"/>
          <xsl:for-each select="element">
            <xsl:call-template name="process-element">
              <xsl:with-param name="path" select="concat($path, '/', @name)"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:element>
      </xsl:when>
      <!-- Other elements are only added if found. -->
      <xsl:otherwise>
        <axsl:apply-templates select="{$path}"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- Process rules. -->

  <xsl:template name="process-rules">
    <xsl:param name="path" select="@name"/>

    <!-- Define elements which do not have selectors. -->
    <!-- NOTE: Duplicating adding-selectors - see above. -->
    <xsl:variable name="adding-selectors" select="count(//selector[@element=current()/@name])"/>

    <xsl:choose>
      <xsl:when test="@type='multiple-choice-value' or @type='multiple-choice-list-value'">
        <!-- Do not match multiple-choice values. -->
      </xsl:when>
      <xsl:when test="$adding-selectors = 0">
        <xsl:for-each select="element">
          <xsl:call-template name="process-rules">
            <xsl:with-param name="path" select="concat($path, '/', @name)"/>
          </xsl:call-template>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- Fill in enumerations. -->

  <xsl:template name="inside-enumeration">
    <xsl:param name="path" select="@name"/>

    <!-- Store multiple-choice selections, if appropriate. -->
    <xsl:if test="../@type='multiple-choice-list'">
      <axsl:variable name="values-{@name}" select="{$path}/@{../attribute/@name}"/>
    </xsl:if>

    <axsl:for-each select="${../@name}/{../@name}/{@name}">
      <axsl:copy>
        <axsl:apply-templates select="@*"/>
        <xsl:if test="@type='multiple-choice-list-value'">
          <axsl:if test="$values-{@name}[string() = current()/@{attribute/@name}]">
            <axsl:attribute name="{@expr-name}">true</axsl:attribute>
          </axsl:if>
        </xsl:if>
      </axsl:copy>
    </axsl:for-each>
  </xsl:template>

</xsl:stylesheet>
